@@ -2,7 +2,7 @@
 * application.cpp: Application class
 *-------------------------------------------------------------------------------
 * (c)1999-2001 VideoLAN
-* $Id: application.cpp,v 1.11 2002/10/07 15:01:21 sam Exp $
+* $Id: application.cpp,v 1.17.4.4 2003/03/06 03:18:20 tooney Exp $
 *
 * Authors: Benoit Steiner <benny@via.ecp.fr>
 *
@@ -26,6 +26,41 @@
 *
 *******************************************************************************/
 
+#define COPYRIGHT \
+"VideoLAN Server - version 0.5.0 - (c)1996-2002 VideoLAN\n"
+
+#define VERSION \
+  COPYRIGHT \
+  "\n" \
+  "This program comes with NO WARRANTY, to the extent permitted by law.\n" \
+  "You may redistribute it under the terms of the GNU General Public " \
+  "License;\n" \
+  "see the file named COPYING for details.\n" \
+  "Written by the VideoLAN team at Ecole Centrale, Paris.\n"
+
+#define HELP \
+  COPYRIGHT \
+  "\n" \
+  " vls [options] target\n" \
+  " \n" \
+  " options: \n" \
+  "                       --version      display version and exit\n" \
+  "   -v                  --verbose      verbosity (v, vv, vvv)\n" \
+  "   -h                  --help         display this help\n" \
+  "   -d <target>         --destination  output \n" \
+  "   -f <file>           --file <file>  configuration file\n" \
+  "   -t <number>         --ttl <number> ttl value\n" \
+  "   -l                  --loop         looping at end of program\n" \
+  "                       --log <file>   log to <file> \n" \
+  " \n" \
+  " target:\n" \
+  "   dvd:<device>     for streaming from a dvd\n" \
+  "   dvd:<dir>        for streaming from a dvd stored on hard drive\n" \
+  "   file:<file>      for streaming from a file\n" \
+  "   dvb:<channel>    for streaming from a dvb channel\n" \
+  "   udp:<ip[:port]>  for streaming in udp to ip \n" \
+  "   rtp:<ip[:port]>  for streaming in rtp to ip \n" \
+  "   file:<filename>  for dumping output to filename \n"
 
 //------------------------------------------------------------------------------
 // Preamble
@@ -43,6 +78,15 @@
 #ifdef SYSLOG
 #include <syslog.h>
 #endif
+#ifdef HAVE_GETOPT_H
+#  include <getopt.h>                                            /* getopt() */
+#else
+#  ifdef WIN32
+#    define __STDC__ 1
+#  endif
+#  include "../extras/getopt.h"
+#endif
+
 #include "common.h"
 #include "debug.h"
 #include "reflect.h"
@@ -89,7 +133,7 @@ C_Application::C_Application(const C_String& strName) : m_strName(strName)
 
   m_bOnStop = false;
 
-  m_iLogFlags = LOG_DBGMSG;
+  m_iLogFlags = LOG_ERRORMSG;
 }
 
 
@@ -117,9 +161,8 @@ int C_Application::Init(int iArgc, char* paArg[])
   // Init the logger
   if(!iRc)
   {
-    C_String strLogFileSetting = m_strName + ".LogFile";
-    C_String strDfltLogFile = m_strName.ToLower() + ".log";
-    C_String strLogFile = GetSetting(strLogFileSetting, strDfltLogFile);
+    C_String strLogFileSetting = "Global.LogFile";
+    C_String strLogFile = GetSetting(strLogFileSetting, "");
 
     if (strLogFile != "")
     {
@@ -129,13 +172,13 @@ int C_Application::Init(int iArgc, char* paArg[])
   }
 
   // Check if System Logging is enabled in the vls.cfg
-  C_String strSystemLog = GetSetting(m_strName + ".SystemLog", "disable");
+  C_String strSystemLog = GetSetting("Global.SystemLog", "disable");
 
   if (strSystemLog == "enable")
     m_iLogFlags |= LOG_SYSTEM;
 
   // Check if screen Logging is enabled in the vls.cfg
-  C_String strScrLog = GetSetting(m_strName + ".ScreenLog", "enable");
+  C_String strScrLog = GetSetting("Global.ScreenLog", "enable");
 
   if (strScrLog == "enable")
     m_iLogFlags |= LOG_SCR;
@@ -340,45 +383,116 @@ int C_Application::InstallSigHandler()
 int C_Application::RetrieveConfig(int iArgc, char* paArg[])
 {
   C_String strCfgFile;
+  C_String strDest;
   int iRc = NO_ERR;
- 
+  int iCommandline=0;
+  int opt;
 
   C_String strName = m_strName.ToLower() + ".cfg";
 
-  if (paArg[1]== NULL)
-    strCfgFile = strName;
-  else
-    strCfgFile = C_String(paArg[1]);
-
  #ifndef _WIN32
-  C_File cFile(strCfgFile);
-  if(!cFile.Exists())
-  {
-    strCfgFile = CONFIG_PATH "/" + strName;
+   strCfgFile = CONFIG_PATH "/" + strName;
+ #else
+   strCfgFile = strName;
  #endif
+ 
+  static const struct option longopts[]={
+    {"verbose", 0 , 0 , 'v'},
+    {"help", 0 , 0, 'h'},
+    {"file", 1, 0, 'f'},
+    {"version", 0, 0, 129},
+    {"destination", 1, 0,'d'},
+    {"ttl", 1, 0, 't'},
+    {"loop", 0,0, 'l'},
+    {"log", 1, 0, 130},
+    {0,0,0,0}
+  };
+  static char shortopts[]="f:d:hvt:l";
+  
+  //Parse command line options
+  while((opt = getopt_long(iArgc, paArg, shortopts, longopts, 0)) != EOF)
+  {
+    switch(opt)
+    {
+      case 'v':
+       if(m_iLogFlags&LOG_NOTEMSG) m_iLogFlags |= LOG_DBGMSG; 
+       if(m_iLogFlags&LOG_WARNMSG) m_iLogFlags |= LOG_NOTEMSG; 
+       if(m_iLogFlags&LOG_ERRORMSG) m_iLogFlags |= LOG_WARNMSG; 
+       break;
+
+      case 129:
+       printf(VERSION);
+       exit(0);
+       break;
+
+      case 'f':
+       strCfgFile = optarg;
+       break;
+
+      case 'h':
+       printf(HELP);
+       exit(0);
+
+      default:
+       if(optarg!=NULL)
+         m_cSettings.AddCmdLineSetting(opt,C_String(optarg));
+       else
+         m_cSettings.AddCmdLineSetting(opt,"");
+       break;
+       
+    }
+  }
+  if(optind<iArgc) //Use cmdline
+  {
+     C_String sTags="";
+     try
+     {
+       if(GetSetting("program1.loop", "false")=="true") sTags+="--loop ";
+       if(GetSetting("output1.protocol", "udp")=="rtp") sTags+="--rtp ";
+       int iTtl = GetSetting("channel1.ttl", "0").ToInt();
+       if(iTtl) sTags=sTags+"--ttl "+iTtl+" ";
+         m_cSettings.AddCmdLineTarget(paArg[optind], sTags);
+     }
+     catch(E_Exception e)
+     {
+       printf("%s\nCommand line launching failed. Aborting.\n",
+		                               e.Dump().GetString());
+       exit(0);
+     }
+     iRc=0;
+  }
+  else //Use configuration file
+  {
+    if(iCommandline)
+    {
+      printf("Command line arguments specified with no target. Aborting.\n");
+      exit(0);
+    }
     C_File cFile(strCfgFile);
     if(!cFile.Exists())
     {
-      printf("No configuration file found\n");
-      iRc = GEN_ERR;
+      strCfgFile = m_strName.ToLower()+".cfg";
+      cFile=C_File(strCfgFile);
+      if(!cFile.Exists())
+      {
+        printf("No configuration file found (%s)\n", strCfgFile.GetString());
+        iRc = GEN_ERR;
+      }
     }
- #ifndef _WIN32
-  }
- #endif
 
-  if(!iRc)
-  {
-    try
+    if(!iRc)
     {
-      m_cSettings.Load(strCfgFile, false);
-    }
-    catch(E_Exception e)
-    {
-      printf("Could not open configuration file: \n%s", e.Dump().GetString());
-      iRc = GEN_ERR;
+      try
+      {
+        m_cSettings.Load(strCfgFile, false);
+      }
+      catch(E_Exception e)
+      {
+        printf("Could not open configuration file: \n%s", e.Dump().GetString());
+        iRc = GEN_ERR;
+      }
     }
   }
-
   return iRc;
 }
 
