@@ -2,9 +2,10 @@
 * dvbinput.cpp: DVB streams
 *-------------------------------------------------------------------------------
 * (c)1999-2002 VideoLAN
-* $Id: dvbinput.cpp,v 1.5 2002/04/02 15:22:08 bozo Exp $
+* $Id: dvbinput.cpp,v 1.21.4.3 2003/03/08 14:03:35 nitrox Exp $
 *
 * Authors: Arnaud de Bossoreille de Ribou <bozo@via.ecp.fr>
+*          Damien Lucas <nitrox@videolan.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
@@ -34,17 +35,23 @@
 #include <fcntl.h>
 #include <sys/ioctl.h>
 
-#include <dvbpsi/dvbpsi.h>
-#include <dvbpsi/descriptor.h>
-#include <dvbpsi/pat.h>
-#include <dvbpsi/pmt.h>
+#ifdef HAVE_DVBPSI_DVBPSI_H
+#   include <dvbpsi/dvbpsi.h>
+#   include <dvbpsi/descriptor.h>
+#   include <dvbpsi/pat.h>
+#   include <dvbpsi/pmt.h>
+#else
+#   include "src/dvbpsi.h"
+#   include "src/descriptor.h"
+#   include "src/tables/pat.h"
+#   include "src/tables/pmt.h"
+#endif
 
-#include <ost/sec.h>
-#include <ost/frontend.h>
-#include <ost/dmx.h>
+#include <DVB.hh>
 
 #include "../../mpeg/mpeg.h"
 #include "../../mpeg/ts.h"
+#include "../../mpeg/rtp.h"
 
 #include "../../server/program.h"
 #include "../../server/buffer.h"
@@ -63,6 +70,7 @@
 
 #include "dvbinput.h"
 
+#define FILELEN 256
 
 //------------------------------------------------------------------------------
 // Library declaration
@@ -97,13 +105,17 @@ C_DvbInput::C_DvbInput(C_Module* pModule,
                                 C_Input(pModule, strName),
                                 C_TsDemux(&m_cTsProvider),
                                 m_cTsProvider(500),
-                                m_cInputProgram(0, "Input DVB " + strName),
+                                m_cInputProgram(/*0,*/ "Input DVB " + strName),
                                 m_cInputBroadcast(&m_cInputProgram, this, NULL),
                                 m_cPatDecoder(&m_cTsProvider, this),
                                 m_cCurrentPat(0, 0, true)
 {
+  dvb = new DVB;
+  m_iGotTpid = 0;            // Did not set the transponder yet
+  m_iDemuxUsageCount = 0;    // Nothing using the demux yet
+  m_bIgnoreTimeout = false;
   m_pConverter = NULL;
-  for(int i =0; i < 64; i++)
+  for(int i =0; i < 512; i++)
     m_iDemuxes[i] = -1;
 }
 
@@ -121,45 +133,83 @@ C_DvbInput::~C_DvbInput()
 //------------------------------------------------------------------------------
 void C_DvbInput::OnInit()
 {
+  int iNumber;
+  C_String strType;
+  C_String dvbrc;
+  char filen[FILELEN];
+  char devname[80];
+
   // Retrieve config
   C_Application* pApp = C_Application::GetApp();
   ASSERT(pApp);
 
-  C_String strNumber = pApp->GetSetting(GetName() + "DeviceNumber", "");
+  iNumber = pApp->GetSetting(GetName() + ".DeviceNumber", "0").ToInt();
+  m_iSendMethod = pApp->GetSetting(GetName() + ".SendMethod", "0").ToInt();
+  m_bIgnoreTimeout = pApp->GetSetting(GetName()+".IgnoreTimeout", "0").ToInt();
+  dvbrc = pApp->GetSetting(GetName()+ ".Dvbrc", "");
+  if(dvbrc.Length() != 0)
+  {
+    strncpy(filen, dvbrc.GetString(), dvbrc.Length()+1);
+  }
+  dvb->init("", "", 0, iNumber);
 
-  m_strSec      = "/dev/ost/sec" + strNumber;
-  m_strFrontend = "/dev/ost/frontend" + strNumber;
-  m_strDemux    = "/dev/ost/demux" + strNumber;
-  m_strDvr      = "/dev/ost/dvr" + strNumber;
+  sprintf(devname, DVR_DEV,   iNumber, 0); m_strDVR   = C_String(devname);
+  sprintf(devname, DEMUX_DEV, iNumber, 0); m_strDEMUX = C_String(devname);
+  sprintf(devname, VIDEO_DEV, iNumber, 0); m_strVIDEO = C_String(devname);
 
-  m_iFrequency = pApp->GetSetting(GetName() + ".Frequency", "0").ToInt();
-  m_iPolarization = pApp->GetSetting(GetName() + ".Polarization", "0").ToInt();
-  m_iSymbolRate = pApp->GetSetting(GetName() + ".SymbolRate", "0").ToInt();
+  //Check whether card has a decoder:
+  //When opening the video device, we should get an error
+  int iDummy=open(m_strVIDEO.GetString(), O_RDWR|O_NONBLOCK);
+  m_bHasDecoder=(iDummy<0) ? false : true;
+  close(iDummy);
+  switch(dvb->front_type)
+  {
+    case FE_QPSK:
+      strType = "DVB-S";
+      break;
+    case FE_QAM:
+      strType= "DVB-C";
+      break;
+    case FE_OFDM:
+      strType = "DVB_T";
+      break;
+    default:
+      throw E_Exception(GEN_ERR, "No DVB card found");
+      break;
+  }
+  Log(m_hLog, LOG_NOTE, strType + " Card registered "+
+                  (m_bHasDecoder ? "with decoder\n" : "without decoder\n"));
 
-  m_iDiSEqC = pApp->GetSetting(GetName() + ".DiSEqC", "0").ToInt();
+  // Enable DVR (sets the DMX_OUT_TS_TAP in the lib)
+  dvb->enable_DVR();
+  if(m_bHasDecoder) dvb->enable_DVR_other();
 
-  m_iLnbLof1 = pApp->GetSetting(GetName() + ".LNB_Lof1", "0").ToInt();
-  m_iLnbLof2 = pApp->GetSetting(GetName() + ".LNB_Lof2", "0").ToInt();
-  m_iLnbSLof = pApp->GetSetting(GetName() + ".LNB_SLof", "0").ToInt();
+  //Get the .dvbrc file
+  if (!get_dvbrc(filen,*dvb,iNumber,FILELEN))
+    throw E_Exception(GEN_ERR, "Unable to find any dvbrc file");
 
-  // Hardware initialization
-  SecControl();
-  SetFrontend();
+  for(int i=0; i<dvb->NumChannel(); i++)
+  {
+    C_String* pStr=new C_String(dvb->chans[i].name);
+    /* Let's replace all spaces by '_' */
+    pStr->Replace(' ', '_');
+    m_vProgramNames.Add(pStr);
+    Log(m_hLog, LOG_NOTE, "Added program '" + *pStr+"'");
+  }
 
   // PAT decoder initialization
   m_cPatDecoder.Attach();
 
-  // Select the PAT PID
-  SelectPid(&m_cPatDecoder, 0x0000, TS_TYPE_NULL);
 
   // Reader
   C_MpegReaderModule* pReaderModule = (C_MpegReaderModule*)
                                 C_Application::GetModuleManager()
                                         ->GetModule("mpegreader",
-                                                    "file");
+                                                    "dvb");
   ASSERT(pReaderModule);
 
-  m_cInputBroadcast.SetOption("filename", m_strDvr);
+  m_cInputBroadcast.SetOption("device", m_strDVR);
+  m_cInputBroadcast.SetOption("IgnoreTimeout", m_bIgnoreTimeout);
   m_pReader = pReaderModule->NewMpegReader(&m_cInputBroadcast);
   ASSERT(m_pReader);
 
@@ -180,14 +230,6 @@ void C_DvbInput::OnInit()
   cConfig.m_pEventHandler = this;
   m_pConverter = pConverterModule->NewMpegConverter(cConfig);
   ASSERT(m_pConverter);
-
-  m_cEndInit.Protect();
-
-  // Launch the demux
-  m_pConverter->Create();
-
-  // Kludge: wait for the first PAT arrival.
-  m_cEndInit.Wait();
 }
 
 
@@ -196,26 +238,25 @@ void C_DvbInput::OnInit()
 //------------------------------------------------------------------------------
 void C_DvbInput::OnDestroy()
 {
-  // Unselect the PAT PID
-  C_TsDemux::Lock();
-  UnselectPid(&m_cPatDecoder, 0x0000);
-  C_TsDemux::UnLock();
 
   // PAT Decoder destruction
   m_cPatDecoder.Detach();
 
   if(m_pConverter)
   {
-    // Stop the input stream
-    try
+    // Stop the input converter if necessary
+    if(m_pConverter->IsRunning())
     {
-      m_pConverter->Stop();
-    }
-    catch(E_Exception e)
-    {
-      m_cEndInit.Release();
-      delete m_pConverter;
-      throw e;
+      try
+      {
+        m_pConverter->Stop();
+      }
+      catch(E_Exception e)
+      {
+        m_cEndInit.Release();
+        delete m_pConverter;
+        throw e;
+      }
     }
 
     delete m_pConverter;
@@ -226,23 +267,21 @@ void C_DvbInput::OnDestroy()
 
 
 //------------------------------------------------------------------------------
-// Get the program table
+// Return the programs read in the dvbrc file
 //------------------------------------------------------------------------------
 C_List<C_Program> C_DvbInput::OnGetAvailablePgrms()
 {
   C_List<C_Program> cPgrmList;
 
   m_cLock.Lock();
-
-  dvbpsi_pat_program_t *pPatProgram =
-                        m_cCurrentPat.GetLowLevelPat()->p_first_program;
-  while(pPatProgram)
-  {
-    C_Program* pProgram = new C_Program(pPatProgram->i_number,
-                                        pPatProgram->i_number);
+  for(int i=0; i<dvb->NumChannel(); i++)
+  { 
+    C_String pStr=dvb->chans[i].name;
+    /* Let's replace all spaces by '_' */
+    pStr.Replace(' ', '_');
+    C_Program* pProgram = new C_Program(pStr, dvb->chans[i].pnr);
     ASSERT(pProgram);
     cPgrmList.PushEnd(pProgram);
-    pPatProgram = pPatProgram->p_next;
   }
 
   m_cLock.UnLock();
@@ -257,52 +296,64 @@ C_List<C_Program> C_DvbInput::OnGetAvailablePgrms()
 void C_DvbInput::OnSelectPid(u16 iPid, u8 iType)
 {
   int i;
-  for(i = 0; (i < 32) && (m_iDemuxes[2 * i] != -1); i++);
+  int iRc;
 
-  if(i < 32)
+  // Look for the first available demux
+  for(i = 0; (i < 256) && (m_iDemuxes[2 * i] != -1); i++);
+
+  // Check for the 256 Pid filters hardware limitation
+  // note that dvb->init open 4 demuxes
+  if(i < 256 - 4)
   {
-    int iFd = open(m_strDemux.GetString(), O_RDWR|O_NONBLOCK);
-    if(iFd < 0)
-      Log(m_hLog, LOG_ERROR, "Unable to open demux");
-    else
+    // TODO: test if it is not a required pid (PCR for example)
+    if((m_iSendMethod == 0) || (iType < TS_TYPE_MPEG2_PRIVATE))
     {
-      struct dmxPesFilterParams pesFilterParams;
 
-      pesFilterParams.pid     = iPid;
-      pesFilterParams.input   = DMX_IN_FRONTEND;
-      pesFilterParams.output  = DMX_OUT_TS_TAP;
-      switch(iType)
+      int iFd = open(m_strDEMUX.GetString(), O_RDWR|O_NONBLOCK);
+      if(iFd < 0)
       {
-      case TS_TYPE_MPEG1_VIDEO:
-      case TS_TYPE_MPEG2_VIDEO:
-        pesFilterParams.pesType = DMX_PES_VIDEO;
-        break;
-      case TS_TYPE_MPEG1_AUDIO:
-      case TS_TYPE_MPEG2_AUDIO:
-        pesFilterParams.pesType = DMX_PES_AUDIO;
-        break;
-      default:
-        pesFilterParams.pesType = DMX_PES_OTHER;
-        break;
+        Log(m_hLog, LOG_ERROR, "Unable to open demux");
       }
-      pesFilterParams.flags  = DMX_IMMEDIATE_START;
-
-      if(ioctl(iFd, DMX_SET_PES_FILTER, &pesFilterParams) < 0)
+      else
       {
-        Log(m_hLog, LOG_ERROR, C_String("Unable to set demux filter for PID ") + iPid);
+        switch(iType)
+        {
+          case TS_TYPE_MPEG1_VIDEO:
+          case TS_TYPE_MPEG2_VIDEO:
+            iRc = dvb->set_vpid_fd(iPid, iFd);
+            break;
+          case TS_TYPE_MPEG1_AUDIO:
+            iRc = dvb->set_apid_fd(iPid, iFd);
+            break;
+          default:
+            iRc = dvb->set_otherpid_fd(iPid, iFd);
+            break;
+        }
+      }
+
+      if(iRc < 0)
+      {
+        Log(m_hLog, LOG_ERROR, C_String("Unable to set demux filter for PID ")
+                  + iPid + C_String("type : ") + iType);
         close(iFd);
       }
       else
       {
-        LogDbg(m_hLog, C_String("Demux filter set for PID ") + iPid);
+        LogDbg(m_hLog, C_String("Demux filter n")+i+ C_String(" type: ") +
+                 iType + C_String(" set for PID ") + iPid);
         m_iDemuxes[2 * i] = iPid;
         m_iDemuxes[2 * i + 1] = iFd;
       }
     }
+    else
+    {
+      LogDbg(m_hLog, C_String("Demux filter not started for PID ")
+                    + iPid + C_String(" ES type: ") + iType);
+    }
   }
   else
   {
-    Log(m_hLog, LOG_ERROR, "To many PID selected");
+     LogDbg(m_hLog, C_String("Too many Demux filters set"));
   }
 }
 
@@ -313,11 +364,12 @@ void C_DvbInput::OnSelectPid(u16 iPid, u8 iType)
 void C_DvbInput::OnUnselectPid(u16 iPid)
 {
   int i;
-  for(i = 0; (i < 32) && (m_iDemuxes[2 * i] != iPid); i++);
+  for(i = 0; (i < 256) && (m_iDemuxes[2 * i] != iPid); i++);
 
-  if(i < 32)
+  if(i < 256)
   {
     ASSERT(m_iDemuxes[2 * i] == iPid);
+    ioctl(m_iDemuxes[2 * i + 1], DMX_STOP);
     close(m_iDemuxes[2 * i + 1]);
     m_iDemuxes[2 * i] = -1;
     LogDbg(m_hLog, C_String("Demux filter unset for PID ") + iPid);
@@ -398,180 +450,98 @@ void C_DvbInput::OnDvbPsiPatEvent(int iEvent)
     // Kludge: signal the first PAT arrival.
     m_cEndInit.Protect();
     m_cEndInit.Signal();
+    m_cEndInit.Release();
   }
 }
 
-
-//------------------------------------------------------------------------------
-// Hardware initialization
-//------------------------------------------------------------------------------
-void C_DvbInput::SecControl()
-{
-  struct secCommand sSCmd;
-  struct secCmdSequence sSCmdSeq;
-  int iSecFd;
-
-  // Open the device
-  iSecFd = open(m_strSec.GetString(), O_RDWR);
-
-  if(iSecFd < 0)
-  {
-    throw E_Exception(GEN_ERR, "Unable to open \"" + m_strSec +
-                      "\": " + GetErrorMsg());
-  }
-
-  // Set the frequency of the transponder, taking into account the
-  // local frequencies of the LNB
-  sSCmdSeq.continuousTone = (m_iFrequency < m_iLnbSLof) ?
-                            SEC_TONE_OFF : SEC_TONE_ON;
-
-  // Set the polarity of the transponder by setting the correct
-  // voltage on the universal LNB
-  sSCmdSeq.voltage = (m_iPolarization) ? SEC_VOLTAGE_18 : SEC_VOLTAGE_13;
-
-  // In case we have a DiSEqC, set it to the correct address
-  sSCmd.type = 0;
-  sSCmd.u.diseqc.addr = 0x10;
-  sSCmd.u.diseqc.cmd = 0x38;
-  sSCmd.u.diseqc.numParams = 1;
-  sSCmd.u.diseqc.params[0] =   0xF0
-                            | ((m_iDiSEqC * 4) & 0x0F)
-                            | (sSCmdSeq.continuousTone == SEC_TONE_ON ? 1 : 0)
-                            | (sSCmdSeq.voltage == SEC_VOLTAGE_18 ? 2 : 0);
-
-  sSCmdSeq.miniCommand = SEC_MINI_NONE;
-  sSCmdSeq.numCommands = 1;
-  sSCmdSeq.commands = &sSCmd;
-
-  // Send the data to the SEC device to prepare the LNB for tuning
-  Log(m_hLog, LOG_NOTE, "Sending data to the SEC");
-
-  if(ioctl(iSecFd, SEC_SEND_SEQUENCE, &sSCmdSeq) < 0)
-  {
-    close(iSecFd);
-    throw E_Exception(GEN_ERR, "Unable to send data to the SEC");
-  }
-
-  Log(m_hLog, LOG_NOTE, "Data successfully sent to the SEC\n");
-
-  close(iSecFd);
-}
-
-
-//------------------------------------------------------------------------------
-// Hardware initialization
-//------------------------------------------------------------------------------
-void C_DvbInput::SetFrontend()
-{
-  FrontendParameters sFeParams;
-  int iFrontFd;
-
-  // Open the device
-  iFrontFd = open(m_strFrontend.GetString(), O_RDWR);
-
-  if(iFrontFd < 0)
-  {
-    throw E_Exception(GEN_ERR, "Unable to open \"" + m_strSec +
-                      "\": " + GetErrorMsg());
-  }
-
-  // Set the frequency of the transponder, taking into account the
-  // local frequencies of the LNB
-  sFeParams.Frequency = (m_iFrequency < m_iLnbSLof) ?
-                        m_iFrequency - m_iLnbLof1 : m_iFrequency - m_iLnbLof2;
-
-  // Set symbol rate and FEC
-  sFeParams.u.qpsk.SymbolRate = m_iSymbolRate;
-  sFeParams.u.qpsk.FEC_inner = FEC_AUTO;
-
-  // Now send it all to the frontend device
-
-  Log(m_hLog, LOG_NOTE, "Sending data to the Frontend");
-
-  if(ioctl(iFrontFd, FE_SET_FRONTEND, &sFeParams) < 0)
-  {
-    close(iFrontFd);
-    throw E_Exception(GEN_ERR, "Unable to send data to the Frontend");
-  }
-
-  Log(m_hLog, LOG_NOTE, "Data successfully sent to the Frontend\n");
-
-  // Check if it worked
-  fd_set sFdSet;
-  struct timeval sTimeVal;
-  FrontendEvent event;
-
-  // poll for Frontend event to check if tuning worked
-  FD_ZERO(&sFdSet);
-  FD_SET(iFrontFd, &sFdSet);
-  sTimeVal.tv_sec = 3;
-  sTimeVal.tv_usec = 0;
-
-  if(select(iFrontFd + 1, &sFdSet, NULL, NULL, &sTimeVal) == 1)
-  {
-    ASSERT(FD_ISSET(iFrontFd, &sFdSet));
-
-    Log(m_hLog, LOG_NOTE, "Frontend: getting event");
-    if(ioctl(iFrontFd, FE_GET_EVENT, &event) == -EBUFFEROVERFLOW)
-    {
-      close(iFrontFd);
-      throw E_Exception(GEN_ERR, "Frontend: unable to get event");
-    }
-
-    Log(m_hLog, LOG_NOTE, "Frontend: event received");
-    switch(event.type)
-    {  
-    case FE_UNEXPECTED_EV:
-      close(iFrontFd);
-      throw E_Exception(GEN_ERR, "Frontend: unexpected event");
-    case FE_FAILURE_EV:
-      close(iFrontFd);
-      throw E_Exception(GEN_ERR, "Frontend: failure event");
-    case FE_COMPLETION_EV:
-      Log(m_hLog, LOG_NOTE, "Frontend: completion is OK");
-    }
-  }
-  else
-  {
-    close(iFrontFd);
-    throw E_Exception(GEN_ERR, "Frontend: time out: " + GetErrorMsg());
-  }
-
-  close(iFrontFd);
-}
-
-
 //------------------------------------------------------------------------------
 // Start the reception of the given program
 //------------------------------------------------------------------------------
 void C_DvbInput::OnStartStreaming(C_Broadcast* pBroadcast)
 {
+
+  // Get the channel Id
+  int iIndex=m_vProgramNames.Find(pBroadcast->GetProgram()->GetName());
+  LogDbg(m_hLog, "DVB Channel found: "+dvb->chans[iIndex].name);
+
+
+  // Lock the demux usage
+  m_cDemuxUsageM.Lock();
+
+  //If we have not already started the demux, do so and wait for the first PAT
+  if (m_iDemuxUsageCount == 0)
+  {
+    // Set the frontend up
+    dvb->SetTP(dvb->chans[iIndex].tpid, dvb->chans[iIndex].satid);
+    dvb->set_front();
+
+    sleep(3);
+
+    //Launch the demux
+    m_pConverter->Create();
+
+    // Add a filter for PAT
+    SelectPid(&m_cPatDecoder, 0x0000, TS_TYPE_NULL);
+
+    // Wait for the first PAT
+    m_cEndInit.Wait();
+    m_cEndInit.Release();
+    m_iGotTpid = dvb->chans[iIndex].tpid; // Remember the transponder
+  }
+  else
+  {
+    //Check that if we have already got one broadcast going this
+    //new one is on the same mux (transponder)
+    if(m_iGotTpid != dvb->chans[iIndex].tpid)
+    {
+      Log(m_hLog, LOG_ERROR, C_String("Attempting to start reception from" \
+          " different transponder.Existing Transponder is") +m_iGotTpid +
+        " New transponder is" + dvb->chans[iIndex].tpid);
+      m_cDemuxUsageM.UnLock();
+      return;
+    }
+  }
+
+  // Update Demux Counter and unlock
+  m_iDemuxUsageCount++;
+  m_cDemuxUsageM.UnLock();
+
+  // Get the program
+  dvbpsi_pat_program_t *pProgram =
+        m_cCurrentPat.GetProgram(dvb->chans[iIndex].pnr);
+
   m_cLock.Lock();
 
-  dvbpsi_pat_program_t *pProgram =
-        m_cCurrentPat.GetProgram(pBroadcast->GetProgram()->GetName().ToInt());
-
+  // And launch the streaming line
   if(pProgram)
   {
+
+
     C_SyncFifo* pBuffer =
         new C_SyncFifo(2 * pBroadcast->GetChannel()->GetBuffCapacity());
 
+
     C_TsStreamer *pStreamer = new C_TsStreamer(m_hLog, pBroadcast,
                                                m_pTsProvider, pBuffer,
                                                m_pEventHandler, false, false);
 
     C_TsMux *pMux = new C_TsMux(m_pTsProvider, this, pBuffer);
 
+
     try
     {
+      u16 iNumber = pBroadcast->GetProgram()->GetName().ToInt();
+
       pStreamer->Create();
 
       pMux->Attach();
 
       pMux->AttachProgram(pProgram->i_number, pProgram->i_pid);
 
-      m_cMuxes.Add(pProgram->i_number, pMux);
-      m_cStreamers.Add(pProgram->i_number, pStreamer);
+      m_cMuxes.Add(iNumber, pMux);
+
+      m_cStreamers.Add(iNumber, pStreamer);
+
     }
     catch(E_Exception e)
     {
@@ -579,6 +549,7 @@ void C_DvbInput::OnStartStreaming(C_Broadcast* pBroadcast)
       delete pMux;
       throw e;
     }
+
   }
   else
   {
@@ -614,6 +585,21 @@ void C_DvbInput::OnStopStreaming(C_Broadcast* pBroadcast)
 {
   m_cLock.Lock();
 
+  //Lock demux counter and decrement usage counter
+  m_cDemuxUsageM.Lock();
+  m_iDemuxUsageCount--;
+
+  // If the usage counter is 0, we have to remove the PAT filter and suspend
+  // the demux
+  if(m_iDemuxUsageCount==0)
+  {
+    //Unset the filter on PAT
+    UnselectPid(&m_cPatDecoder, 0x0000);
+    m_pConverter->Stop();
+  }
+
+  m_cDemuxUsageM.UnLock();
+
   u16 iNumber = pBroadcast->GetProgram()->GetName().ToInt();
 
   C_TsMux *pMux = m_cMuxes.Remove(iNumber);
@@ -640,4 +626,14 @@ void C_DvbInput::OnStopStreaming(C_Broadcast* pBroadcast)
   delete pStreamer;
 }
 
+//------------------------------------------------------------------------------
+// Update configuration dynamically
+//------------------------------------------------------------------------------
+void C_DvbInput::OnUpdateProgram(C_String strProgram,
+                                     C_String strFileName, C_String strType)
+{
+}
 
+void C_DvbInput::OnDeleteProgram(C_String strProgram)
+{
+}
