@@ -2,7 +2,7 @@
 * buffer.cpp: Buffers management
 *-------------------------------------------------------------------------------
 * (c)1999-2001 VideoLAN
-* $Id: buffer.cpp,v 1.4 2002/07/12 18:09:36 massiot Exp $
+* $Id: buffer.cpp,v 1.4.4.1 2003/03/06 03:08:25 tooney Exp $
 *
 * Authors: Benoit Steiner <benny@via.ecp.fr>
 *
@@ -82,7 +82,12 @@ void C_SyncFifo::HandlePacket(C_TsPacket* pPacket)
 
   // Warn the waiting threads if any that they have data to pop
   iRc = m_cNotEmptySignal.Post();
+
+#ifdef WIN32
+  ASSERT(iRc);
+#else
   ASSERT(!iRc);
+#endif
 
 /*
   unsigned int iSize = m_pFifo->Size();
@@ -119,7 +124,13 @@ C_TsPacket* C_SyncFifo::Pop()
 
   // Warn the waiting threads if any that they have data to push
   iRc = m_cNotFullSignal.Post();
+  
+#ifdef WIN32
+  ASSERT(iRc);
+#else
   ASSERT(!iRc);
+#endif
+
   
   return pPacket;
 
